﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDeckerBus
{
    internal class SetBusesGeneric<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        private int BusyPlaces = 0;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBusesGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <returns></returns>
        public int Insert(T bus)
        {
            return Insert(bus, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="bus">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T bus, int position)
        {
            if (position < 0 || position >= _maxCount || BusyPlaces == _maxCount) return -1;

            BusyPlaces++;
            _places.Insert(position, bus);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= _maxCount) return null;
            T savedBus = _places[position];
            _places.RemoveAt(position);
            return savedBus;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= _maxCount) return null;
                return _places[position];
            }
            set
            {
                Insert(value, position);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetBuses()
        {
            foreach (var bus in _places)
            {
                if (bus != null)
                {
                    yield return bus;
                }
                else
                {
                    yield break;
                }
            }
        }

    }
}
